#!/opt/puppetlabs/puppet/bin/ruby
# frozen_string_literal: true

require 'puppet'
require 'open3'
require 'fileutils'

begin
  require_relative '../../ruby_task_helper/files/task_helper.rb'
  require_relative '../lib/task_helper'
rescue LoadError
  # include location for unit tests
  require 'fixtures/modules/ruby_task_helper/files/task_helper.rb'
  require 'fixtures/modules/comply/lib/task_helper'
end

# RemoveAssessor task
class RemoveAssessor < TaskHelper
  def task(number_to_keep: 1, **_kwargs)
    install_path, = return_install_directories
    old_versions = []
    invalid_versions = []
    real_install_path = File.realpath(install_path)
    parent_dir = File.dirname(real_install_path)
    current_dir = File.basename(parent_dir)
    unwanted_dir = ['.', '..']

    assessors_path = return_assessors_path

    directories = Dir.entries(assessors_path).select do |entry|
      path = File.join(assessors_path, entry)
      File.directory?(path) && entry != '.' && entry != '..'
    end

    if number_to_keep > directories.size
      $stdout.puts "(Warning) number_to_keep: #{number_to_keep} is greater than the number of available directories: #{directories.size}"
      return
    end

    Dir.foreach(assessors_path) do |f|
      next if unwanted_dir.include?(f) || f == current_dir

      if Gem::Version.correct?(f)
        if Gem::Version.new(f) < Gem::Version.new(current_dir)
          old_versions << f
        elsif Gem::Version.new(f) > Gem::Version.new(current_dir)
          invalid_versions << f
        end
      else
        invalid_versions << f
      end
    end

    sorted = old_versions.sort_by { |v| Gem::Version.new(v) }.reverse
    sorted += invalid_versions

    no_of_old_values = sorted.length
    Dir.foreach(assessors_path) do |f|
      next if unwanted_dir.include?(f)
      full_path = File.join(assessors_path, f)
      if number_to_keep > 1
        ((number_to_keep - 1)..(no_of_old_values - 1)).each do |i|
          if f == sorted[i]
            $stdout.puts "Removed: #{f}"
            FileUtils.rm_rf(full_path)
          end
        end
      elsif f != current_dir
        FileUtils.rm_rf(full_path)
        $stdout.puts "Removed: #{f}"
      end
    end

    $stdout.printf 'Keeping:' + ' '
    Dir.foreach(assessors_path) do |f|
      next if unwanted_dir.include?(f)
      $stdout.printf f + ' '
    end
  end
end

if $PROGRAM_NAME == __FILE__
  RemoveAssessor.run
end
